/***************************************************************************
                                arc_vfs.cpp
                             -------------------
    begin                : Thu May 4 2000
    copyright            : (C) 2000 by Shie Erlich & Rafi Yanai
 ---------------------------------------------------------------------------
  Description 
 ***************************************************************************

  A 

     db   dD d8888b. db    db .d8888.  .d8b.  d8888b. d88888b d8888b.
     88 ,8P' 88  `8D 88    88 88'  YP d8' `8b 88  `8D 88'     88  `8D
     88,8P   88oobY' 88    88 `8bo.   88ooo88 88   88 88ooooo 88oobY'
     88`8b   88`8b   88    88   `Y8b. 88~~~88 88   88 88~~~~~ 88`8b
     88 `88. 88 `88. 88b  d88 db   8D 88   88 88  .8D 88.     88 `88.
     YP   YD 88   YD ~Y8888P' `8888Y' YP   YP Y8888D' Y88888P 88   YD

                                                     S o u r c e    F i l e

 ***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#include "krusader.h"
#include "arc_vfs.h"
#include <qregexp.h>
#include <qdir.h>
#include <qdatetime.h>
#include <kprocess.h>
#include "krusader.h"
#include <qfileinfo.h>
#include <kio/jobclasses.h>
#include <kstddirs.h>

//constructor
arc_vfs::arc_vfs(QString origin,QString type,QWidget* panel,bool write):
  vfs(panel),arcFile(origin),changed(false),prefix(""){
  supportMoveFrom = true;
  krConfig->setGroup("Advanced");
  supportMoveTo = krConfig->readBoolEntry("Allow Move Into Archive",_MoveIntoArchive);

  // find the please wait gif
  plzwait = KGlobal::dirs()->findResource("appdata","plzwait.png");

  // set the writable attribute
	QFileInfo temp(origin);
	struct stat stat_p;
  stat(origin,&stat_p);
  mode_t m = stat_p.st_mode;
  if (geteuid() == 0) isWritable = true;
  else if( m&0002 ) isWritable = true;
  else if( temp.groupId()==getegid() && m&0020 ) isWritable = true;
  else if( temp.ownerId()==geteuid() && m&0200 ) isWritable = true;
  else isWritable = false;
  isWritable = ( write && isWritable ); 	

  // tell the user to wait
  SplashScreen Screen(krApp->mainView,0,plzwait,0);
  Screen.show();

  vfs_type = type;
  // create the temp dir..
  tmpDir = origin;//.right(15);
  for(int i=tmpDir.find('/',0) ; i != -1; i=tmpDir.find('/',i+1))
    tmpDir.replace(i,1,"");
  for(int i=tmpDir.find(' ',0) ; i != -1; i=tmpDir.find(' ',i+1))
    tmpDir.replace(i,1,"");

  tmpDir.sprintf(tmpDir+"%d",kapp->random()*kapp->random()%100000);
  tmpDir = "/tmp/"+tmpDir;
  QDir().mkdir(tmpDir,true);

  krConfig->setGroup("External");
  // fill the command options
  if(type == "-tgz" || type == "gzip" ){
    cmd = krConfig->readEntry("GZip Path",_GZipPath);
    listCmd = "-l";
    delCmd  = "none";
    addCmd  = "gzip -c";
    getCmd  = "-dc";
    supportCopyTo = false;
    supportDelete = false;
    supportMoveTo = false;
  }
  if(type == "zip2" || type =="-tbz"){
    cmd = krConfig->readEntry("BZ2 Path",_BZ2Path);
    listCmd = "none";
    delCmd  = "none";
    addCmd  = "bzip2 -c";
    getCmd  = "-dc";
    supportCopyTo = false;
    supportDelete = false;
    supportMoveTo = false;
  }
  if(type == "-tar"){
    cmd = krConfig->readEntry("Tar Path",_TarPath);
    listCmd = " --force-local -tvf";
    delCmd  = cmd+" --delete --force-local -f";
    addCmd  = cmd+" --force-local -uvf";
    getCmd  = " -xvf";
    supportCopyTo = true;
    supportDelete = true;
  }
  if(type == "-zip"){
    cmd = krConfig->readEntry("UnZip Path",_UnZipPath);
    listCmd = "-ZTs ";
    QString zipcmd = krConfig->readEntry("Zip Path",_ZipPath);
    delCmd  = zipcmd+" -rd";
    addCmd  = zipcmd+" -ry";
    getCmd  = " -o";
    supportCopyTo = false;
    supportDelete = false;
  }
  if(type == "-rpm"){
    cmd = "rpm";//krConfig->readEntry("UnZip Path",_UnZipPath);
    listCmd = " --dump -lpq ";
    delCmd  = "none";
    addCmd  = "none";
    getCmd  = " --force-local -idmvF";
    supportCopyTo = false;
    supportDelete = false;
    supportMoveTo = false;
    isWritable    = false;
  }

  getDirs();
}

arc_vfs::~arc_vfs(){
  // tell the user to wait
  SplashScreen Screen(krApp->mainView,0,plzwait,0);

  if(vfs_type == "-tbz" || vfs_type == "-tgz" || vfs_type == "gzip" || vfs_type == "zip2"){
    vfile* vf = vfs_getFirstFile();
    QFileInfo qfi(tmpDir+"/"+vf->vfile_getName());
    if(qfi.exists() && ( qfi.size() != vf->vfile_getSize() )){
      Screen.show();
      KShellProcess proc;
      QString save = getcwd(0,0);
      chdir(tmpDir);
      proc << addCmd << tmpDir+"/"+vfs_filesP->first()->vfile_getName()+" > "+"\""+arcFile+"\"";
      proc.start(KProcess::Block);
      chdir(save);
    }
  } else{
    Screen.show();
    repack("");
  }
  // delete the temp dir
  KShellProcess proc;
  proc<<"rm"<<"-rf"<<tmpDir;
  proc.start(KProcess::Block);
}	

void arc_vfs::getDirs(){
  // clear the dir list
  dirList.clear();

  // prepare the first dir entry - the "" entry
  arc_dir *tempdir = new arc_dir("");
  vfs_filesP = &(tempdir->entries);
  dirList.append(tempdir);
  if(listCmd != "none"){
    // write the temp file
    KShellProcess proc;
    proc << cmd << listCmd << "\""+arcFile+"\"" <<" > " << tmpDir+"/tempfilelist";
    proc.start(KProcess::Block);

    // parse the temp file
    QFile temp(tmpDir+"/tempfilelist");
    temp.open(IO_ReadOnly);
    QString line;
    if(vfs_type == "-tgz" || vfs_type == "gzip" || vfs_type == "-zip" )
    temp.readLine(line,10000);  // skip the first line - it's garbage
    while(temp.readLine(line,10000) != -1)
      parseLine(line.stripWhiteSpace());
    temp.close();
    QDir().remove(tmpDir+"/tempfilelist");
  }
  else parseLine(""); // bzip2
}


// copy a file to the vfs (physical)
void arc_vfs::vfs_addFiles(KURL::List *fileUrls,bool move,QWidget* toNotify,QString dir){
  if (addCmd == "none") return;

  // get the path inside the archive
  QString path = vfs_origin.right((vfs_origin.length()-vfs_origin.findRev('\\'))-1);
  path = path+"/";
  if(dir != "" ) dir = "/"+dir;
  if(path.left(1) != "/") path = "/"+path;
  QDir().mkdir(tmpDir+path+dir);

  KIO::Job* job = new KIO::CopyJob(*fileUrls,tmpDir+path+dir,move,false,false);
  new KRprogress(job);
  connect(job,SIGNAL(result(KIO::Job*)),this,SLOT(vfs_refresh()) );
  if(move) // notify the other panel
   connect(job,SIGNAL(result(KIO::Job*)),toNotify,SLOT(slotPanelRefresh()) );
}
	

// remove a file from the vfs (physical)
void arc_vfs::vfs_delFiles(QValueList<QString> *fileNames){
  // tell the user to wait
  SplashScreen Screen(krApp->mainView,0,plzwait,0);
  Screen.show();

  // get the path inside the archive
  QString path = vfs_origin.right((vfs_origin.length()-vfs_origin.findRev('\\'))-1);
  path = path+"/";
  if(path.left(1) == "/") path.remove(0,1);

  KShellProcess proc1 , proc2;
  proc1 <<  delCmd << "\""+arcFile+"\"";
  proc2 << "rm -rf";
  QValueList<QString>::Iterator name;
  for(name = fileNames->begin(); name != fileNames->end() ; ++name){
    if(vfs_type == "-zip")
      proc1 << path+(*name)+"/*"; // allows to delete directorys
    proc1 << path+(*name);
    proc2 << tmpDir+"/"+path+(*name);
  }
	proc1.start(KProcess::Block);
	proc2.start(KProcess::Block);
	if(!inRefresh) vfs_refresh();
}	

// return a path to the file
QString arc_vfs::rpm_getFile(QString name){
  if(!cpioExist){
    KShellProcess cpio;
    cpio << "rpm2cpio"<<arcFile+" > "+tmpDir+"/temp.cpio";
    cpio.start(KProcess::Block);
    cpioExist = true;
  }
  return "";
}
	
// return a path to the file
QString arc_vfs::vfs_getFile(QString name){
  // change dir to the temp dir
  QString save = getcwd(0,0);
	chdir(tmpDir);
	// tell the user to wait
  SplashScreen Screen(krApp->mainView,0,plzwait,0);
  Screen.show();

  if (vfs_type == "-rpm") rpm_getFile(name);

  // extract the file
  KShellProcess proc;
  // get the current file path
  QString path = vfs_origin.right((vfs_origin.length()-vfs_origin.findRev('\\'))-1);
  if(path.left(1)=="/") path.remove(0,1);
  findArcDir(path)->extFiles.append(name);
  if(path != "") path = path+"/";
  if(vfs_search(name)->vfile_isDir()) name = name+"/";

  // dont unpack the same file twice
  if(QDir(tmpDir).exists(path+name)) return tmpDir+"/"+path+name;

  if(vfs_type == "-tbz" || vfs_type == "-tgz" || vfs_type == "gzip" || vfs_type == "zip2")
    proc << cmd << getCmd << "\""+arcFile+"\"" << " > "+tmpDir+"/"+name;
  else if (vfs_type == "-rpm")
    proc << "cpio" << getCmd << tmpDir+"/temp.cpio" << path+name;
  else{
    proc << cmd << getCmd << "\""+arcFile+"\"" << path+name;
    if(vfs_type == "-zip")
      proc << path+name+"*"; // allows to unpack directorys
  }
  proc.start(KProcess::Block);

  // restore dir
	chdir(save);
  return tmpDir+"/"+path+name;
}



// make dir
void arc_vfs::vfs_mkdir(QString name){
  QString path = vfs_origin.right((vfs_origin.length()-vfs_origin.findRev('\\'))-1);
  if(path.left(1)=="/") path.remove(0,1);
  if(path != "") path = path+"/";

  QDir(tmpDir).mkdir(path+name);
  vfs_refresh();
}
	
// rename file
void arc_vfs::vfs_rename(QString fileName,QString newName){
  KURL::List temp(vfs_getFile(fileName));
  QString path = vfs_origin.right((vfs_origin.length()-vfs_origin.findRev('\\'))-1);
  if(path.left(1)=="/") path.remove(0,1);
  if(path != "") path = path+"/";

  QDir(tmpDir).mkdir(path);

  KIO::Job* job = new KIO::CopyJob(temp,tmpDir+path+"/"+newName,true,false,false);
  connect(job,SIGNAL(result(KIO::Job*)),this,SLOT(vfs_refresh()) );
}

bool arc_vfs::vfs_refresh(const QString& origin){
  // check that the archive still there
  QFileInfo qfi(arcFile);
  if (!qfi.exists()) return false;

  inRefresh = true;
  vfs_origin = origin;
  emit startUpdate();

  // tell the user to wait (only if operation is longer than 1 sec)
  SplashScreen Screen(krApp->mainView,0,plzwait,0);
  QTimer::singleShot(1000, &Screen, SLOT(show()) );

  // get the directory...
  QString path = origin.right((origin.length()-origin.findRev('\\'))-1);
  if(path.left(1) =="/") path.remove(0,1);

  QDir(tmpDir).mkdir(path);
  repack("");
  if (changed){  // rescan dirs only if needed
    getDirs();
    changed = false;
  }

  vfs_filesP = findDir(path);

  inRefresh = false;
  emit updated();
  emit endUpdate();
  return true;
}

// update the file list
void arc_vfs::update(){}	

// service functions
QString arc_vfs::nextWord(QString &s,char d) {
  s=s.stripWhiteSpace();
  int j=s.find(d,0);
  QString temp=s.left(j); // find the leftmost word.
  s.remove(0,j);
  return temp;
}

void arc_vfs::repack(QString dir_name, bool root){
  if(vfs_type == "-tbz" || vfs_type == "-tgz" ||
     vfs_type == "gzip" || vfs_type == "zip2" ||
     vfs_type == "-rpm" )
    return;

  QString origin = vfs_origin; // save the current origin
  vfs_origin = vfs_origin.left(vfs_origin.find('\\')+1)+dir_name;

  // set the right path inside the archive
  vfs_filesP=findDir(dir_name);
  if (vfs_filesP == 0) {
    dir_name = dir_name+"/";
    vfs_filesP=findDir(dir_name);
    if (vfs_filesP == 0) return;
  }

  QDir dir(tmpDir+"/"+dir_name,QString::null, QDir::Name, QDir::All|QDir::Hidden);
  arc_dir* ad = findArcDir(dir_name);
  QFileInfoList* fileList = const_cast<QFileInfoList*>(dir.entryInfoList());


  // delete from the archive files that were unpacked and deleted
  QValueList<QString>::Iterator name;
  if (ad != 0){
    for(name=ad->extFiles.begin(); name != ad->extFiles.end();){
      if(dir.exists(*name)) name = ad->extFiles.remove(name);
      else ++name;
    }
    if(!ad->extFiles.isEmpty()){
      vfs_delFiles(&(ad->extFiles)); // remove the files from the archive
      changed=true;                  // rescan archive
    }
  }
  // delete from tmpDir files that are already archived
  for(QFileInfo* qfi=fileList->first(); qfi!=0;qfi=fileList->next()){
    if(qfi->fileName()!=".." && qfi->fileName()!="."){
      if(qfi->isDir()){
        repack(dir_name+"/"+qfi->fileName(),false);
        vfs_filesP=findDir(dir_name);
      } else {
        vfile * vf =vfs_search(qfi->fileName());
        struct stat stat_p;
        stat("\""+(qfi->fileName())+"\"" ,&stat_p);
        // if the file exist remove it - we will repack it (so changed == true)
        if(vf != 0 && vf->vfile_getDateTime()==dateTime2QString(qfi->lastModified()))
          dir.remove(qfi->fileName()); else changed = true;
      }
    }
  }
  if (ad != 0) ad->extFiles.clear(); // clear the extFiles list
  vfs_origin = origin;  // restore the origin

  // finaly repack tmpDir
  if(root){
    KShellProcess proc;
    QString save = getcwd(0,0);
    chdir(tmpDir);
    proc << addCmd << "\""+arcFile+"\"" << prefix+"*";
    proc.start(KProcess::Block);
    chdir(save);
  }
}

QList<vfile>* arc_vfs::findDir(QString name){
  for(arc_dir* temp = dirList.first();temp != 0 ; temp = dirList.next()){
    if(temp->name == name) return &(temp->entries);
  }
  return 0;
}

arc_dir* arc_vfs::findArcDir(QString name){
for(arc_dir* temp = dirList.first();temp != 0 ; temp = dirList.next()){
    if(temp->name == name) return temp;
  }
  return 0;
}

QString arc_vfs::changeDir(QString name){
  if(name.left(2) == "./") {
    prefix = "./";
  }

  if(!name.contains('/')){
    vfs_filesP = findDir("");
    return name;
  }
  // seperate the path from the name
  QString path = name.left(name.findRev('/'));
  name = name.mid(name.findRev('/')+1,name.length());
  // see if the path exists
  if ((vfs_filesP=findDir(path)) == 0){
   //create a new dir entry
   QFileInfo qfi("\""+arcFile+"\"");
   QString Pname = path.mid(path.findRev('/')+1,path.length());
   if(Pname == "") return name;
   struct stat stat_p;
   stat("\""+arcFile+"\"" ,&stat_p);
   QString perm = mode2QString(stat_p.st_mode);
   perm[0] = 'd';
   vfile* vf=new vfile(Pname,0,perm,dateTime2QString(qfi.lastModified()),false,
                 qfi.owner(),qfi.group(),"inode/directory","",stat_p.st_mode );
   // add  dirs if needed
   changeDir(path);

   vfile* vf2 = vfs_search(Pname);
   if(vf2 != 0) vfs_removeFromList(vf2);
   vfs_addToList(vf);

   // add a new arc_dir
   dirList.append(new arc_dir(path));
   vfs_filesP = findDir(path);
  }
  return name;
}

void arc_vfs::parseLine(QString line){
  // parse gziped files
  if(vfs_type == "-tgz" || vfs_type == "gzip"){
    nextWord(line);
    long size = nextWord(line).toLong();
    nextWord(line);
    QString name = nextWord(line,'\n');
    if(name.contains('/')) name = name.mid(name.findRev('/')+1,name.length());
    QFileInfo qfi("\""+arcFile+"\"");
    struct stat stat_p;
    stat("\""+arcFile+"\"" ,&stat_p);
    QString mime = KMimeType::findByURL( "/"+name,0,true,true)->name();
    vfile* vf=new vfile(name,size,mode2QString(stat_p.st_mode),dateTime2QString(qfi.lastModified()),
                        false,qfi.owner(),qfi.group(),mime,"",stat_p.st_mode );
    vfs_addToList(vf);
  }

  // parse bzip2ed files
  if(vfs_type == "-tbz" || vfs_type == "zip2"){
    QFileInfo qfi("\""+arcFile+"\"");
    QString name = qfi.fileName();
    name = name.left(name.findRev('.'));
    long size = qfi.size();
    struct stat stat_p;
    stat("\""+arcFile+"\"" ,&stat_p);
    QString mime = KMimeType::findByURL( "/"+name,0,true,true)->name();
    vfile* vf=new vfile(name,size,mode2QString(stat_p.st_mode),dateTime2QString(qfi.lastModified()),
                        false,qfi.owner(),qfi.group(),mime,"",stat_p.st_mode );
    vfs_addToList(vf);
  }

  // parse tar files
  if(vfs_type == "-tar"){
    QString perm = nextWord(line);
    QString temp = nextWord(line);
    QString owner = temp.left(temp.findRev('/'));
    QString group = temp.mid(temp.find('/')+1,temp.length());
    long size = nextWord(line).toLong();
    temp = nextWord(line);
    QString dateTime = temp.mid(8,2)+"/"+temp.mid(5,2)+"/"+temp.mid(2,2)+
                       " "+nextWord(line).left(5);
    QString name = nextWord(line);
    name = changeDir(name);
    if(name.length() < 1) return;
    QString mime = KMimeType::findByURL( "/"+name,0,true,true)->name();
    vfile* vf=new vfile(name,size,perm,dateTime,false,owner,group,mime,"",0);
    vfile* vf2 = vfs_search(name);
    if(vf2 != 0) vfs_removeFromList(vf2);
    vfs_addToList(vf);
  }

  // parse zipped files
  if(vfs_type == "-zip"){
    QString perm = nextWord(line);
    if(perm.length() != 10) perm = perm.at(0)+"rwxrwxrwx";
    if (nextWord(line)=="files,") return;
    nextWord(line);
    long size = nextWord(line).toLong();
    nextWord(line);nextWord(line);
    QString temp = nextWord(line);
    QString dateTime = temp.mid(6,2)+"/"+temp.mid(4,2)+"/"+temp.mid(2,2)+
                       " "+temp.mid(9,2)+":"+temp.mid(11,2);
    QString name = nextWord(line);
    name = changeDir(name);
    if(name.length() < 1) return;
    QString mime = KMimeType::findByURL( "/"+name,0,true,true)->name();
    vfile* vf=new vfile(name,size,perm,dateTime,false,geteuid(),getegid(),mime,"",0);
    vfs_addToList(vf);
  }

  // parse rpm packages
  if(vfs_type == "-rpm"){
    QString name = nextWord(line);
    if ( name.left(1) == "/" ) name.remove(0,1);
    long size = nextWord(line).toLong();
    long time = nextWord(line).toLong();
    QString dateTime = time2QString(time);
    nextWord(line);
    QString perm = mode2QString(nextWord(line).toLong());
    QString owner = nextWord(line);
    QString group = nextWord(line);
    name = changeDir(name);
    if(name.length() < 1) return;
    QString mime = KMimeType::findByURL( "/"+name,0,true,true)->name();
    vfile* vf=new vfile(name,size,perm,dateTime,false,owner,group,mime,"",0);


    vfile* vf2 = vfs_search(name);
    if(vf2 != 0) return;
    vfs_addToList(vf);
  }
}
