/***************************************************************************
                                arc_vfs.h
                             -------------------
    begin                : Thu May 4 2000
    copyright            : (C) 2000 by Shie Erlich & Rafi Yanai
 ---------------------------------------------------------------------------
  Description
 ***************************************************************************

  A

     db   dD d8888b. db    db .d8888.  .d8b.  d8888b. d88888b d8888b.
     88 ,8P' 88  `8D 88    88 88'  YP d8' `8b 88  `8D 88'     88  `8D
     88,8P   88oobY' 88    88 `8bo.   88ooo88 88   88 88ooooo 88oobY'
     88`8b   88`8b   88    88   `Y8b. 88~~~88 88   88 88~~~~~ 88`8b
     88 `88. 88 `88. 88b  d88 db   8D 88   88 88  .8D 88.     88 `88.
     YP   YD 88   YD ~Y8888P' `8888Y' YP   YP Y8888D' Y88888P 88   YD

                                                     H e a d e r    F i l e

 ***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#ifndef ARC_VFS_H
#define ARC_VFS_H

#include <vfs.h>
#include <qvaluestack.h>
#include <kprocess.h>

class arc_dir{
public:
  arc_dir(QString _name){
    name = _name;
    entries.setAutoDelete(true);
  }
  QString name;         // the name of the dir
  QList<vfile> entries; // the file and dir in this dir
  QValueList<QString> extFiles; // the name of files unpack from this dir
};

class arc_vfs : public vfs  {
		Q_OBJECT
public:
	arc_vfs(QString origin,QString type,QWidget* panel,bool write);
 ~arc_vfs();
	
	// copy a file to the vfs (physical)
	void    vfs_addFiles(KURL::List *fileUrls,bool move,QWidget* toNotify=0,QString dir = "");	
	// remove a file from the vfs (physical)
	void 		vfs_delFiles(QValueList<QString> *fileNames);	
	// return a path to the file
	QString vfs_getFile(QString name);
	QString rpm_getFile(QString name);
	// make dir
	void vfs_mkdir(QString name);
	// rename file
	void vfs_rename(QString fileName,QString newName);
	
public slots:
	// actually reads files and stats
	bool vfs_refresh(const QString& origin);
	inline  bool vfs_refresh(){return vfs_refresh(vfs_origin);}
  void repack(QString dir_name, bool root = true);
  // update the file list
	void update();	
	
protected:
  QString tmpDir;   // the temp directory tha archive is using
  QString arcFile;  // the archive file URL
  bool inRefresh;   // the refresh function flag
  bool changed;     // true if repack changed the archive
  QList<arc_dir> dirList;
  QString plzwait;

  void getDirs();   // fill the dir list
  QList<vfile>* findDir(QString name);
  arc_dir* findArcDir(QString name);
  QString nextWord(QString &s, char d=' ');
  QString changeDir(QString name);
  // misc parsing functions
  void parseLine(QString line);

  QString prefix;   // to use with ./ tar files
  bool cpioExist;   // used with the rpm_getFile()

  QString cmd;      // the archiver main command
  QString listCmd;  // the file listing option
  QString delCmd;   // the delete option
  QString addCmd;   // the add files option
  QString getCmd;   // the extract files option
};

#endif
