/***************************************************************************
                                kfilelist.cpp
                             -------------------
    Version							 : Milestone 1
    begin                : Thu May 4 2000
    copyright            : (C) 2000 by Shie Erlich & Rafi Yanai
 ---------------------------------------------------------------------------
  Description
 ***************************************************************************

  A

     db   dD d8888b. db    db .d8888.  .d8b.  d8888b. d88888b d8888b.
     88 ,8P' 88  `8D 88    88 88'  YP d8' `8b 88  `8D 88'     88  `8D
     88,8P   88oobY' 88    88 `8bo.   88ooo88 88   88 88ooooo 88oobY'
     88`8b   88`8b   88    88   `Y8b. 88~~~88 88   88 88~~~~~ 88`8b
     88 `88. 88 `88. 88b  d88 db   8D 88   88 88  .8D 88.     88 `88.
     YP   YD 88   YD ~Y8888P' `8888Y' YP   YP Y8888D' Y88888P 88   YD

                                                     S o u r c e    F i l e

 ***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "kfilelist.h"
#include "krusader.h"
#include "krlistitem.h"
#include "defaults.h"
#include <qdir.h>
#include <qdragobject.h>
#include <qtimer.h>
#include <qpainter.h>

KFileList::KFileList(QWidget *parent, const char *name ) :
  QListView(parent,name) {
  dragSX=dragSY=0;
  dragState=nothing;
  pressX=pressY=0;
  stillPressed=false;
  setMouseTracking(false);
  setDragAutoScroll(true);
  toolTip=0;
  toolTipItem=0;
}

/////////////////////////////////// slots ////////////////////////////////////////
void KFileList::select(QString filter) {
  krConfig->setGroup("General");
  //////////////////////////////
  QListViewItem *temp=currentItem();
  QListViewItem *iterator=firstChild();
  while (iterator!=0) {
    if (iterator->text(1)==QString("<DIR>") &&
        !krConfig->readBoolEntry("Mark Dirs",_MarkDirs)) {
      iterator=iterator->itemBelow();
      continue;
    }
    if (QDir::match(filter,iterator->text(0))) {
      iterator->setSelected(true);
      iterator=iterator->itemBelow();
    } else iterator=iterator->itemBelow();
  }
  triggerUpdate();
  ensureItemVisible(temp);
  emit selectionChanged();
}

void KFileList::unselect(QString filter) {
  QListViewItem *temp=currentItem();
  QListViewItem *iterator=firstChild();
  while (iterator!=0) {
    if (iterator->isSelected() && QDir::match(filter,iterator->text(0)))
      iterator->setSelected(false);
    iterator=iterator->itemBelow();
  }
  triggerUpdate();
  ensureItemVisible(temp);
  emit selectionChanged();
}

void KFileList::invertSelection() {
  QListViewItem *temp=currentItem();
  QListViewItem *iterator=firstChild();
  while (iterator!=0) {
    if (iterator->isSelected()) {
      iterator->setSelected(false);
      iterator=iterator->itemBelow();
      continue;
    }
    if (iterator->text(1)==QString("<DIR>") && !krConfig->readBoolEntry("Mark Dirs",_MarkDirs)); //NOP
      else iterator->setSelected(true);
    iterator=iterator->itemBelow();
  }
  triggerUpdate();
  ensureItemVisible(temp);
  emit selectionChanged();
}

void KFileList::keyPressEvent(QKeyEvent *e) {
  QKeyEvent *ne;

  if ( !e || !firstChild() )	return; // subclass bug
  switch (e->key()) {
    case Key_Enter  :
    case Key_Return :
      QListView::keyPressEvent(e);
      return;
    case Key_Up :
      ne=new QKeyEvent(QEvent::KeyPress,e->key(),e->ascii(),ControlButton);
      QListView::keyPressEvent(ne);
      break;
    case Key_Down :
      ne=new QKeyEvent(QEvent::KeyPress,e->key(),e->ascii(),ControlButton);
      QListView::keyPressEvent(ne);
      break;
    case Key_Home :
      ne=new QKeyEvent(QEvent::KeyPress,e->key(),e->ascii(),ControlButton);
      QListView::keyPressEvent(ne);
      break;
    case Key_End :
      ne=new QKeyEvent(QEvent::KeyPress,e->key(),e->ascii(),ControlButton);
      QListView::keyPressEvent(ne);
      break;
    case Key_PageUp :
      ne=new QKeyEvent(QEvent::KeyPress,e->key(),e->ascii(),ControlButton);
      QListView::keyPressEvent(ne);
      break;
    case Key_PageDown :
      ne=new QKeyEvent(QEvent::KeyPress,e->key(),e->ascii(),ControlButton);
      QListView::keyPressEvent(ne);
      break;
    case Key_Backspace :      // dir up
      if (e->state()==ControlButton) Krusader::App->root(); // ROOT
        else
      if (e->state()==0) Krusader::App->dirUp();            // dirUp
      return;
    case Key_QuoteLeft :     // home
      Krusader::App->home();
      return;
    case Key_Insert :         // mark/unmark current
      markCurrent();
      return;
    case Key_Delete :         // kill file
      Krusader::App->mainView->slotDeleteFiles();
      return;
    default:
      QListView::keyPressEvent(e);
      return;
  }
  delete ne;
}


void KFileList::markCurrent() {
	QListViewItem *iterator=currentItem();
	iterator->setSelected(!iterator->isSelected());
	setCurrentItem(iterator->itemBelow());
	triggerUpdate();
}

void KFileList::contentsMouseMoveEvent(QMouseEvent *e) {
  stillPressed=false;
  if (e->state() & LeftButton) {
    if (dragState==dragging) return;
    if (dragState==pending) // let's check how far the mouse moved
      if ( e->x() > dragSX+8 || e->x() < dragSX-8 ||
           e->y() > dragSY+8 || e->y() < dragSY-8 ) {
         dragState=dragging;
         dragSX=0;dragSY=0;
         if (e->state() & ShiftButton) emit letsDrag(1); // move drag
          else emit letsDrag(0);                         // copy/link drag
         return;
      } else return;
    // if we got here, dragState==nothing, so let's start a pseudo-drag
    dragSX=e->x(); dragSY=e->y();
    dragState=pending;
  } else
  if (e->state() & RightButton) QListView::contentsMouseMoveEvent(e);
    else
  if (e->state() & MidButton) {}  // future use ???
}

void KFileList::viewportMouseMoveEvent(QMouseEvent *e) {
  //stillPressed=false;
  if (e->state()!=0) QListView::viewportMouseMoveEvent(e);  // a button is pressed
  // a simple move with no button pressed
  QListViewItem *i=itemAt(e->pos());  // are we pointing on something ?
  if (!i) return;                     // if not, exit.
  emit onItem(i);                     // otherwise, let the world know!
 	if (toolTipItem==i) return;         // don't delete tooltip until we leave the item
 	if (toolTip!=0) {
 	  delete toolTip;
 	  toolTip=0;
 	  toolTipItem=0;
 	}
/* 	int width=QFontMetrics(font()).width(i->text(0));
 	int height=QFontMetrics(font()).height();
 	if (width+20>columnWidth(0)) {
    toolTip=new QWidget(this,0,WStyle_NoBorder|WStyle_Customize);
    toolTipItem=i;
    QLabel *l=new QLabel(i->text(0),toolTip);
    //l->setBackgroundColor(yellow);
    l->setFont(font());
    l->resize(width,height+6);
    toolTip->resize(l->size());
    toolTip->move(
              mapToParent(QPoint(itemRect(i).x()+25,itemRect(i).y()-i->height()-3))
             );
    toolTip->show(); */
//	}
}

void KFileList::viewportMousePressEvent(QMouseEvent *e) {
  if (e->button()==RightButton) { // for right-click only
    // record mouse location for future reference
    pressX=e->pos().x(); pressY=e->pos().y();
    stillPressed=false;
    // after 1/2 a second, we will check if we need to pop a rightclick menu
    QTimer::singleShot( 250, this, SLOT(checkForRightClickMenu()));
    // for now, call the original function to allow swushing through the list
  }
  // anyway, call the original function to allow swushing through the list
  QListView::viewportMousePressEvent(e);
}

void KFileList::viewportMouseReleaseEvent(QMouseEvent *e) {
  if (e->button()==RightButton) // for right-click only
    stillPressed=true;
  else QListView::viewportMouseReleaseEvent(e);
}

// checks if the user has held down the right button for more that 1/2 a second,
// if so, emit a rightClickMenu signal
void KFileList::checkForRightClickMenu() {
  if (stillPressed) {// if the right-button is still pressed
    QListViewItem* i=itemAt(QPoint(pressX,pressY));
    stillPressed=false;
    if (!i) return;
    QPoint temp=mapToGlobal(QPoint(pressX,pressY));
    emit rightClickMenu(i,temp);
  }
}
