/***************************************************************************
                                kmountman.h
                             -------------------
    version              : Milestone 1
    begin                : Thu May 4 2000
    copyright            : (C) 2000 by Shie Erlich & Rafi Yanai
 ---------------------------------------------------------------------------
  Description: Krusader's very own mount manager!
  KMountMan reads /etc/fstab file on creation and creates the filesystems[]
  array, which is the list of potentially mounted filesystems.
  it than allows mounting/unmounting at will and gives information.
  It relies on the ability to read /etc/fstab and to use "df".
  It has 2 flags: operational and ready. if KMountMan is operational, one
  can poll on the ready flag. otherwise, no need to wait, it won't do. :-(
 ***************************************************************************

  A

     db   dD d8888b. db    db .d8888.  .d8b.  d8888b. d88888b d8888b.
     88 ,8P' 88  `8D 88    88 88'  YP d8' `8b 88  `8D 88'     88  `8D
     88,8P   88oobY' 88    88 `8bo.   88ooo88 88   88 88ooooo 88oobY'
     88`8b   88`8b   88    88   `Y8b. 88~~~88 88   88 88~~~~~ 88`8b
     88 `88. 88 `88. 88b  d88 db   8D 88   88 88  .8D 88.     88 `88.
     YP   YD 88   YD ~Y8888P' `8888Y' YP   YP Y8888D' Y88888P 88   YD

                                                     H e a d e r    F i l e

 ***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#ifndef KMOUNTMAN_H
#define KMOUNTMAN_H

// QT includes
#include <qobject.h>
#include <qstring.h>
#include <qframe.h>
#include <qlist.h>
// KDE includes
#include <kprocess.h>
#include <kjanuswidget.h>
#include <kio/jobclasses.h>
// krusader includes
#include <stdlib.h>
#include <math.h>
#include <kmountmangui.h>

// forward definitions
class fsData;

class KMountMan : public QObject {
  Q_OBJECT
  friend class KMountManGUI;
public:
	KMountMan();
	~KMountMan();
  fsData getData(QString path);    // returns info about the whereabouts of path
  inline bool ready()  		 { return Ready; 	     }
  inline bool operational() { return Operational; }
  inline void mainWindow() { new KMountManGUI(); }
  QString followLink(QString source);
  fsData* location(QString name);
  void mount(fsData *p);
  void unmount(fsData *p);
  void autoMount(QString path);

protected slots:
  void parseDfData(QString filename);  // parse a FULL list of filesystems
  void parseDf(QString filename);      // parse a single call for DF
  void mountResult(KIO::Job*);
  void forceUpdate();
  void toggleMount(QString device);

signals:
  void updated();

protected:
  bool createFilesystems(); // potential filesystems from /etc/fstab
  void updateFilesystems(); // actually mounted systems using DF -T -P

private:
QString nextWord(QString &s,char c);

private:
  QList<fsData> filesystems;
  int noOfFilesystems;
  bool Ready;      // true if MountMan finished all preprocessing
  bool Operational;// if false, something went terribly wrong on startup
  fsData *localDf; // used for moving around information
};

// Data container for a single-filesystem data
// maximum size supported is 2GB of 1kb blocks == 2048GB, enough.
/////////////////////////////////////////////////////////////////
class fsData {
public:
  fsData() : Name(0), Type(0), MntPoint(0), TotalBlks(0),
                     UsedBlks(0), Mounted(false) {}

  // get information
  inline QString name()     { return Name;                            }
  inline QString shortName(){ return Name.right(Name.length()-Name.find("/",1)-1); }
  inline QString type()     { return Type;                            }
  inline QString mntPoint() { return MntPoint;                        }
  inline long  totalBytes() { return TotalBlks*1024;                  }
  inline long   usedBytes() { return UsedBlks*1024;                   }
  inline long   freeBytes() { return (TotalBlks-UsedBlks)*1024;       }
  //////////////////// insert a good round function here /////////////////
  int    usedPerct() { float res=((float)UsedBlks)/((float)TotalBlks)*100;
											 if ((res - (int)res) > 0.5) return (int)res + 1;
											 	else return (int)res;                  }
  inline bool     mounted() { return Mounted;                         }

  // set information
  inline void setName(QString n_)     { Name=n_;      }
  inline void setType(QString t_)     { Type=t_;      }
  inline void setMntPoint(QString m_) { MntPoint=m_;  }
  inline void setTotalBlks(long t_)   { TotalBlks=t_; }
  inline void setUsedBlks(long u_)    { UsedBlks=u_;  }
  inline void setMounted(bool m_)     { Mounted=m_;   }

private:
  QString Name;       // i.e: /dev/cdrom
  QString Type;       // i.e: iso9600
  QString MntPoint;   // i.e: /mnt/cdrom
  long    TotalBlks;  // measured in 1024bytes per block
  long    UsedBlks;
  bool    Mounted;    // true if filesystem is mounted
};

#endif
