/***************************************************************************
                                krusader.cpp
                             -------------------
    Version							 : Milestone 1
    begin                : Thu May 4 2000
    copyright            : (C) 2000 by Shie Erlich & Rafi Yanai
 ---------------------------------------------------------------------------
  Description 
 ***************************************************************************

  A 

     db   dD d8888b. db    db .d8888.  .d8b.  d8888b. d88888b d8888b.
     88 ,8P' 88  `8D 88    88 88'  YP d8' `8b 88  `8D 88'     88  `8D
     88,8P   88oobY' 88    88 `8bo.   88ooo88 88   88 88ooooo 88oobY'
     88`8b   88`8b   88    88   `Y8b. 88~~~88 88   88 88~~~~~ 88`8b
     88 `88. 88 `88. 88b  d88 db   8D 88   88 88  .8D 88.     88 `88.
     YP   YD 88   YD ~Y8888P' `8888Y' YP   YP Y8888D' Y88888P 88   YD

                                                     S o u r c e    F i l e

 ***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// KDE includes
#include <kprocess.h>

// QT includes
#include <qmessagebox.h>
#include <qpixmap.h>
#include <qstringlist.h>

// Krusader includes
#include "krusader.h"

Krusader* Krusader::App=0;

// construct the views, statusbar and menu bars and prepare Krusader to start
Krusader::Krusader() : KTMainWindow()
{
  // create the "krusader"
  App=this;
  // create config file
  config=kapp->config();
  QString message;
  switch (config->getConfigState()) {
    case KConfigBase::NoAccess :
      message="Krusader's configuration file can't be found.\nDefault values will be used.";
      break;
    case KConfigBase::ReadOnly :
      message="Krusader's configuration file is in READ ONLY mode (why is that???)\nChanged values will not be saved";
      break;
    case KConfigBase::ReadWrite :
      message="";
     break;
  }
  if (message!="") {
    QMessageBox::critical(0,"A Krusader Error", message);
  }

  // create the main view and set it
  mainView=new KrusaderView(this);
	
	// create a status bar
	status=new KrusaderStatus(this);
	setStatusBar(status);
		
	// add a toolbar
	toolbar=new KrusaderToolBar(this);
	addToolBar(toolbar);
	
	// create the Krusader Menu and set it as the main menu
  mainMenu=new KrusaderMenu(this);
  setMenu(mainMenu);

	disableFutureItems();

  setView(mainView);
  config->setGroup("Private");
  resize(krConfig->readSizeEntry("Start Size",_StartSize));
  move(krConfig->readPointEntry("Start Position",_StartPosition));
  show();

  connect(kapp,SIGNAL(aboutToQuit()),this,SLOT(saveSettings()));
}

Krusader::~Krusader(){}

void Krusader::statusBarUpdate(QString mess) {
  // change the message on the statusbar for 2 seconds
  statusBar()->message(mess,2000);
}

void Krusader::disableFutureItems() {
	MDISABLE( ID_FILES_PACK			   );
  MDISABLE( ID_FILES_UNPACK			 );
  MDISABLE( ID_FILES_TESTARCHIVE );
  MDISABLE( ID_FILES_COMPARE	   );
  MDISABLE( ID_FILES_CALCULATE   );
  MDISABLE( ID_FILES_PRINT			 );

// COMMANDS menu IDs
  //MDISABLE( ID_COMMANDS_SEARCH   );
  MDISABLE(	ID_COMMANDS_SYSINFO  );
  MDISABLE(	ID_COMMANDS_SYNC     );
  MDISABLE(	ID_COMMANDS_BOOKMAN  );
  MDISABLE(	ID_COMMANDS_TERMINAL );
  MDISABLE(	ID_COMMANDS_FTPCONNECT );
  MDISABLE(	ID_COMMANDS_FTPNEWCONNECT );
  MDISABLE(	ID_COMMANDS_FTPDISCONNECT );

// VIEW menu IDs
  MDISABLE(	ID_VIEW_QUICKVIEW );
  MDISABLE(	ID_VIEW_ALLFILES  );
  MDISABLE(	ID_VIEW_EXEC      );
  MDISABLE(	ID_VIEW_CUSTOM    );
  MDISABLE(	ID_VIEW_NAME      );
  MDISABLE(	ID_VIEW_EXTENTION );
  MDISABLE(	ID_VIEW_TIME      );
  MDISABLE(	ID_VIEW_SIZE      );
  MDISABLE(	ID_VIEW_REVERSE   );

// TOOLS menu IDs
  MDISABLE( ID_TOOLS_ADD      );

// OPTIONS menu IDs
  MDISABLE( ID_OPTIONS_TOOLSMENUSETUP );
}

void Krusader::search() {
  KShellProcess proc;
	proc << "kfind";
	proc.start(KProcess::DontCare);
}

///////////////////////////////////////////////////////////////////////////
//////////////////// implementation of slots //////////////////////////////
///////////////////////////////////////////////////////////////////////////

void Krusader::slotFilesCallback(int id_) {
	switch (id_) {
		case ID_FILES_QUIT :
			kapp->quit();
			break;
		case ID_FILES_PROPERTIES :
		  properties();
		  break;
		default:
			break;
	}
}

void Krusader::slotMarkCallback(int id_) {
	switch (id_) {
		case ID_MARK_SALL :
			markAll();
			break;
		case ID_MARK_UALL :
			unmarkAll();
			break;
		case ID_MARK_SGROUP :
			markGroup();
			break;
		case ID_MARK_UGROUP :
			unmarkGroup();
			break;
		case ID_MARK_INVERT :
			invert();
			break;
		default:
			break;
	}
}

void Krusader::slotCommandsCallback(int id_) {
	switch (id_) {
		case ID_COMMANDS_SEARCH :
			search();
			break;
		default :
			break;
	}
}

void Krusader::slotViewCallback(int id_) {
  switch (id_) {
    case ID_VIEW_FULL :
      mainView->setListView();
      break;
    case ID_VIEW_TREE :
      mainView->setTreeView();
      break;
    default:
      break;
  }
}

void Krusader::slotToolsCallback(int id_) {
  switch (id_) {
    case ID_TOOLS_MOUNTMAN :
      runMountMan();
      break;
    default:
      break;
  }
}

void Krusader::slotOptionsCallback(int id_) {
  switch (id_) {
    case ID_OPTIONS_KRUSADERSETUP:
      runKonfigurator();
      break;
    default:
      break;
  }
}

void Krusader::slotStatusCallback(int id_) {
	switch (id_) {
		case ID_FILES_PROPERTIES :
				statusBarUpdate(i18n(S_FILES_PROPERTIES));
				break;
		case ID_FILES_PACK :
				statusBarUpdate(i18n(S_FILES_PACK));
				break;
		case ID_FILES_UNPACK :
				statusBarUpdate(i18n(S_FILES_UNPACK));
				break;
		case ID_FILES_TESTARCHIVE :
				statusBarUpdate(i18n(S_FILES_TESTARCHIVE));
				break;
		case ID_FILES_COMPARE :
				statusBarUpdate(i18n(S_FILES_COMPARE));
				break;
		case ID_FILES_CALCULATE :
				statusBarUpdate(i18n(S_FILES_CALCULATE));
				break;
		case ID_FILES_PRINT :
				statusBarUpdate(i18n(S_FILES_PRINT));
				break;
		case ID_FILES_QUIT :
				statusBarUpdate(i18n(S_FILES_QUIT));
				break;
    case ID_MARK_SGROUP :
        statusBarUpdate(i18n(S_MARK_SGROUP));
        break;
    case ID_MARK_UGROUP :
        statusBarUpdate(i18n(S_MARK_UGROUP));
        break;
    case ID_MARK_SALL :
        statusBarUpdate(i18n(S_MARK_SALL));
        break;
    case ID_MARK_UALL :
        statusBarUpdate(i18n(S_MARK_UALL));
        break;
    case ID_MARK_INVERT :
        statusBarUpdate(i18n(S_MARK_INVERT));
        break;
    case ID_COMMANDS_SEARCH :
        statusBarUpdate(i18n(S_COMMANDS_SEARCH));
        break;
    case ID_COMMANDS_SYSINFO :
        statusBarUpdate(i18n(S_COMMANDS_SYSINFO));
        break;
    case ID_COMMANDS_SYNC :
        statusBarUpdate(i18n(S_COMMANDS_SYNC));
        break;
    case ID_COMMANDS_BOOKMAN :
        statusBarUpdate(i18n(S_COMMANDS_BOOKMAN));
        break;
    case ID_COMMANDS_TERMINAL :
        statusBarUpdate(i18n(S_COMMANDS_TERMINAL));
        break;
    case ID_COMMANDS_FTPCONNECT :
        statusBarUpdate(i18n(S_COMMANDS_FTPCONNECT));
        break;
    case ID_COMMANDS_FTPNEWCONNECT :
        statusBarUpdate(i18n(S_COMMANDS_FTPNEWCONNECT));
        break;
    case ID_COMMANDS_FTPDISCONNECT :
        statusBarUpdate(i18n(S_COMMANDS_FTPDISCONNECT));
        break;
    case ID_VIEW_FULL :
        statusBarUpdate(i18n(S_VIEW_FULL));
        break;
    case ID_VIEW_TREE :
        statusBarUpdate(i18n(S_VIEW_TREE));
        break;
    case ID_VIEW_QUICKVIEW :
        statusBarUpdate(i18n(S_VIEW_QUICKVIEW));
        break;
    case ID_VIEW_ALLFILES :
        statusBarUpdate(i18n(S_VIEW_ALLFILES));
        break;
    case ID_VIEW_EXEC :
        statusBarUpdate(i18n(S_VIEW_EXEC));
        break;
    case ID_VIEW_CUSTOM :
        statusBarUpdate(i18n(S_VIEW_CUSTOM));
        break;
    case ID_VIEW_NAME :
        statusBarUpdate(i18n(S_VIEW_NAME));
        break;
    case ID_VIEW_EXTENTION :
        statusBarUpdate(i18n(S_VIEW_EXTENTION));
        break;
    case ID_VIEW_TIME :
        statusBarUpdate(i18n(S_VIEW_TIME));
        break;
    case ID_VIEW_SIZE :
        statusBarUpdate(i18n(S_VIEW_SIZE));
        break;
    case ID_VIEW_REVERSE :
        statusBarUpdate(i18n(S_VIEW_REVERSE));
        break;
    case ID_TOOLS_ADD :
        statusBarUpdate(i18n(S_TOOLS_ADD));
        break;		
    case ID_TOOLS_MOUNTMAN :
        statusBarUpdate(i18n(S_TOOLS_MOUNTMAN));
        break;		
   case ID_OPTIONS_KRUSADERSETUP :
        statusBarUpdate(i18n(S_OPTIONS_KRUSADERSETUP));
        break;		
   case ID_OPTIONS_TOOLSMENUSETUP :
        statusBarUpdate(i18n(S_OPTIONS_TOOLSMENUSETUP));
        break;		
		default :
				break;
	}
}
	
void Krusader::saveSettings() {
  config->setGroup("General");
  if (config->readBoolEntry("Remember Position",_RememberPos)) {
    config->setGroup("Private");
    config->writeEntry("Start Position", pos());
    config->writeEntry("Start Size", size());
  }
  // delete all vfs records...
  mainView->left->cleanUp();
  mainView->right->cleanUp();
}

void Krusader::addBookmark() {
  krConfig->setGroup("Bookmarks");
  QStringList list=krConfig->readListEntry("Links");
  if (list.count()==0) list << _Links;
  QString path=mainView->activePanel->virtualPath;
  if (path.contains('*')>0) return;
  QStringList::Iterator it=list.begin();
  while (it!=list.end()) {
    if (*it==path) return;
    ++it;
  }
  list << path; // add the new bookmark
  krConfig->writeEntry("Links",list);
}

void Krusader::toggleToolButton(int id,bool on) {
  toolbar->toggleToolButton(id,on);
}

void Krusader::runMountMan() {
  if (mountMan.operational()) {
    while (!mountMan.ready());
    mountMan.mainWindow();
  }
}
