/***************************************************************************
                                krusadermenu.cpp
                             -------------------
    Version							 : Milestone 1
    begin                : Thu May 4 2000
    copyright            : (C) 2000 by Shie Erlich & Rafi Yanai
 ---------------------------------------------------------------------------
 The krusader menubar and all it's pulldown menus are defined here.
 ***************************************************************************

  A 

     db   dD d8888b. db    db .d8888.  .d8b.  d8888b. d88888b d8888b.
     88 ,8P' 88  `8D 88    88 88'  YP d8' `8b 88  `8D 88'     88  `8D
     88,8P   88oobY' 88    88 `8bo.   88ooo88 88   88 88ooooo 88oobY'
     88`8b   88`8b   88    88   `Y8b. 88~~~88 88   88 88~~~~~ 88`8b
     88 `88. 88 `88. 88b  d88 db   8D 88   88 88  .8D 88.     88 `88.
     YP   YD 88   YD ~Y8888P' `8888Y' YP   YP Y8888D' Y88888P 88   YD

                                                     S o u r c e    F i l e

 ***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// KDE includes
#include <ktmainwindow.h>
#include <klocale.h>
#include <kapp.h>
#include <kiconloader.h>

// QT includes
#include <qpixmap.h>
#include <qiconset.h>
#include <qfont.h>

// Krusader includes
#include "krusadermenu.h"
#include "krusader.h"
#include "defaults.h"

KrusaderMenu::KrusaderMenu(QWidget *parent, const char *name ) : KMenuBar(parent,name) {
	krConfig->setGroup("Look&Feel");
	////////////////////////////////
	// set the desired font for the main menu.
  QFont tempFont=krConfig->readFontEntry("Menu Font",_MenuFont);
  setFont(tempFont);

  // the FILES menu:
	files = new QPopupMenu();
	files->setFont(tempFont);
		// ... insert the items
  files->insertItem(i18n("&Properties"), ID_FILES_PROPERTIES);
	files->insertItem(ICON(package_closed_xpm),i18n("&Pack"), ID_FILES_PACK);
	files->insertItem(ICON(package_open_xpm),i18n("&Unpack"), ID_FILES_UNPACK);
	files->insertItem(i18n("Test Archi&ve(s)"), ID_FILES_TESTARCHIVE);
	files->insertItem(i18n("Compare b&y content"), ID_FILES_COMPARE);
	files->insertItem(i18n("Calculate &Occupied Space"), ID_FILES_CALCULATE);
	files->insertItem(i18n("&Print"), ID_FILES_PRINT);
	files->insertSeparator();
  files->insertItem(ICON(krusader24x24_xpm),i18n("&Quit"), ID_FILES_QUIT);
    // ... set the keyboard shortcuts
  files->setAccel(ALT+Key_P,ID_FILES_PROPERTIES);
  files->setAccel(ALT+Key_F5,ID_FILES_PACK);
  files->setAccel(ALT+Key_F9,ID_FILES_UNPACK);
  files->setAccel(ALT+Key_S,ID_FILES_CALCULATE);
		// ... connect the entire menu to it's callback function	
  connect(files, SIGNAL(activated(int)), parent, SLOT(slotFilesCallback(int)));
	connect(files, SIGNAL(highlighted(int)), parent, SLOT(slotStatusCallback(int)));
		// ... insert the whole menu into the menubar
  insertItem( i18n("&Files"), files );

	// the MARK menu
	mark = new QPopupMenu();
	mark->setFont(tempFont);
		// ... insert the items
	mark->insertItem(i18n("Select &Group"), ID_MARK_SGROUP);
	mark->insertItem(i18n("&Unselect &Group"), ID_MARK_UGROUP);
	mark->insertItem(i18n("&Select All"), ID_MARK_SALL);
	mark->insertItem(i18n("U&nselect All"), ID_MARK_UALL);
	mark->insertItem(i18n("&Invert Selection"), ID_MARK_INVERT);
	  // ... set the keyboard shortcuts
	mark->setAccel(CTRL+Key_Plus,ID_MARK_SGROUP);
	mark->setAccel(CTRL+Key_Minus,ID_MARK_UGROUP);
	mark->setAccel(ALT+Key_Asterisk,ID_MARK_INVERT);
	mark->setAccel(ALT+Key_Plus,ID_MARK_SALL);
	mark->setAccel(ALT+Key_Minus,ID_MARK_UALL);
	  // ... connect the entire menu to it's callback function
	connect(mark, SIGNAL(activated(int)), parent, SLOT(slotMarkCallback(int)));
	connect(mark, SIGNAL(highlighted(int)), parent, SLOT(slotStatusCallback(int)));
	  // ... insert the whole menu into the menubar
	insertItem( "&Mark", mark);
	
	// the COMMANDS menu
	commands = new QPopupMenu();
	  // ... insert the items
	commands->setFont(tempFont);
	commands->insertItem(ICON(unknown_xpm),i18n("&Search"), ID_COMMANDS_SEARCH);
	commands->insertItem(ICON(info_xpm),i18n("System &Information"), ID_COMMANDS_SYSINFO);
	commands->insertItem(i18n("S&yncronize Dirs"), ID_COMMANDS_SYNC);
	commands->insertItem(i18n("&Bookmark Manager"), ID_COMMANDS_BOOKMAN);
	commands->insertSeparator();
	commands->insertItem(ICON(terminal_xpm),i18n("&Terminal"), ID_COMMANDS_TERMINAL);
	commands->insertSeparator();
	commands->insertItem(ICON(ftp_xpm),i18n("&FTP Connect"), ID_COMMANDS_FTPCONNECT);
	commands->insertItem(ICON(ftp_xpm),i18n("FT&P New Connection"), ID_COMMANDS_FTPNEWCONNECT);
	commands->insertItem(i18n("FTP Disc&onnect"), ID_COMMANDS_FTPDISCONNECT);
	  // ... set the keyboard shortcuts
	commands->setAccel(ALT+Key_F7, ID_COMMANDS_SEARCH);
	commands->setAccel(CTRL+Key_Slash, ID_COMMANDS_BOOKMAN);
	commands->setAccel(CTRL+Key_T, ID_COMMANDS_TERMINAL);
	commands->setAccel(CTRL+Key_F, ID_COMMANDS_FTPCONNECT);
	commands->setAccel(CTRL+Key_N, ID_COMMANDS_FTPNEWCONNECT);
	commands->setAccel(SHIFT+CTRL+Key_F, ID_COMMANDS_FTPDISCONNECT);
	  // ... connect the entire menu to it's callback function
	connect(commands, SIGNAL(activated(int)), parent, SLOT(slotCommandsCallback(int)));
	connect(commands, SIGNAL(highlighted(int)), parent, SLOT(slotStatusCallback(int)));
    // ... insert the thing into the menubar
	insertItem("&Commands",commands);
	// the VIEW menu
	view = new QPopupMenu();
	view->setFont(tempFont);	
	  // ... insert the items
	view->insertItem(i18n("&Full"), ID_VIEW_FULL);
	view->insertItem(i18n("&Tree"), ID_VIEW_TREE);
	view->insertItem(i18n("&Quickview Panel"), ID_VIEW_QUICKVIEW);
	view->insertSeparator();
	view->insertItem(i18n("&All Files"), ID_VIEW_ALLFILES);
	view->insertItem(i18n("&Executables"), ID_VIEW_EXEC);
	view->insertItem(i18n("&Custom"), ID_VIEW_CUSTOM);
	view->insertSeparator();
	view->insertItem(i18n("&Name"), ID_VIEW_NAME);
	view->insertItem(i18n("Ti&me"), ID_VIEW_TIME);
	view->insertItem(i18n("&Size"), ID_VIEW_SIZE);
	view->insertSeparator();
	view->insertItem(i18n("&Reversed Order"), ID_VIEW_REVERSE);
    // ... set the keyboard shortcuts
  view->setAccel(ALT+Key_1, ID_VIEW_FULL);
  view->setAccel(ALT+Key_2, ID_VIEW_TREE);
  view->setAccel(CTRL+Key_Q, ID_VIEW_QUICKVIEW);
  view->setAccel(SHIFT+Key_F10, ID_VIEW_ALLFILES);
  view->setAccel(SHIFT+Key_F11, ID_VIEW_EXEC);
  view->setAccel(SHIFT+Key_F12, ID_VIEW_CUSTOM);
  view->setAccel(SHIFT+Key_F3, ID_VIEW_NAME);
  view->setAccel(SHIFT+Key_F4, ID_VIEW_TIME);
  view->setAccel(SHIFT+Key_F5, ID_VIEW_SIZE);
    // ... connect the menu to the callback function
	connect(view, SIGNAL(activated(int)), parent, SLOT(slotViewCallback(int)));
	connect(view, SIGNAL(highlighted(int)), parent, SLOT(slotStatusCallback(int)));
    // ... insert the thing into the menubar
	insertItem("&View",view);
	
	// the TOOLS menu
  tools = new QPopupMenu();
	tools->setFont(tempFont);
    // ... insert the items
  tools->insertItem(ICON(mountMan_xpm),i18n("&MountMan"), ID_TOOLS_MOUNTMAN);
  tools->insertSeparator();
	tools->insertItem(ICON(tools_xpm),i18n("&Add a new tool"), ID_TOOLS_ADD);
  tools->insertSeparator();
    // ... accel
  tools->setAccel(ALT+Key_Slash, ID_TOOLS_MOUNTMAN);
    // ... connect the menu to the callback function
	connect(tools, SIGNAL(activated(int)), parent, SLOT(slotToolsCallback(int)));
	connect(tools, SIGNAL(highlighted(int)), parent, SLOT(slotStatusCallback(int)));
    // ... insert the thing into the menubar
  insertItem("&Tools",tools);
	
	// the CONFIGURATION menu
	options = new QPopupMenu();
	options->setFont(tempFont);	
	  // ... insert the items
	options->insertItem(i18n("&Konfigurator"), ID_OPTIONS_KRUSADERSETUP);
	options->insertItem(i18n("&Tools Menu Setup"), ID_OPTIONS_TOOLSMENUSETUP);
	  // ... connect the menu
	connect(options, SIGNAL(activated(int)), parent, SLOT(slotOptionsCallback(int)));
	connect(options, SIGNAL(highlighted(int)), parent, SLOT(slotStatusCallback(int)));
    // ... insert the whole thing	
	insertItem("&Options",options);
	// the HELP menu	
	  // ... it uses the standard KDE help menu
  help = ((KTMainWindow*)parent)->helpMenu(
  				"Krusader\nMilestone 2\n\n(c) Shie Erlich & Rafi Yanai\
  				 \n\nPreview Version");
	insertItem("&Help",help);
	
}
KrusaderMenu::~KrusaderMenu(){
}


