/***************************************************************************
                                kvfspanel.h
                             -------------------
    Version							 : Milestone 1
    begin                : Thu May 4 2000
    copyright            : (C) 2000 by Shie Erlich & Rafi Yanai
 ---------------------------------------------------------------------------
  Description
 ***************************************************************************

  A

     db   dD d8888b. db    db .d8888.  .d8b.  d8888b. d88888b d8888b.
     88 ,8P' 88  `8D 88    88 88'  YP d8' `8b 88  `8D 88'     88  `8D
     88,8P   88oobY' 88    88 `8bo.   88ooo88 88   88 88ooooo 88oobY'
     88`8b   88`8b   88    88   `Y8b. 88~~~88 88   88 88~~~~~ 88`8b
     88 `88. 88 `88. 88b  d88 db   8D 88   88 88  .8D 88.     88 `88.
     YP   YD 88   YD ~Y8888P' `8888Y' YP   YP Y8888D' Y88888P 88   YD

                                                     H e a d e r    F i l e

 ***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#ifndef KVFSPANEL_H
#define KVFSPANEL_H

// KDE includes
#include <kpropsdlg.h>
#include <kfileitem.h>
#include <kurl.h>

// QT includes
#include <qwidget.h>
#include <qlabel.h>
#include <qlayout.h>
#include <qstring.h>
#include <qpushbutton.h>
#include <qpixmap.h>
#include <qtoolbutton.h>
#include <qpopupmenu.h>
#include <qdir.h>
#include <qpixmapcache.h>
#include <qiconset.h>
#include <qstack.h>
#include <qtimer.h>

// Krusader includes
#include "normal_vfs.h"
#include "resources.h"
#include "kicons.h"
#include "kfilelist.h"
#include "krdialogs.h"
#include "panelfunc.h"
#include "kmountman.h"

class KVFSPanel : public QWidget  {
	Q_OBJECT
friend class ListPanelFunc;
friend class TreePanelFunc;
public:
	         KVFSPanel(QWidget *parent,const char *name ) : QWidget(parent,name){}
	virtual ~KVFSPanel(){}
	
	virtual void start(bool left) = 0; // refresh the panel for the 1st time
	// needed for correct resizing of the panel
	void resizeEvent ( QResizeEvent *e );
	// service function for 1st time initiation
	inline void setOther(KVFSPanel *panel) { otherPanel=panel; }
	virtual QString getPath()= 0;
	virtual void cleanUp(){}

protected:
  virtual void dragEnterEvent ( QDragEnterEvent * ) = 0;
  virtual void dragLeaveEvent ( QDragLeaveEvent * ) = 0;
  virtual void dragMoveEvent  ( QDragMoveEvent * )  = 0;
  virtual void dropEvent      ( QDropEvent * )      = 0;

signals:
	void signalStatus(QString msg);
	void cmdLineUpdate(QString p);	// emitted when we need to update the command line
  void activePanelChanged(KVFSPanel *p);
  void finishedDragging();

public slots:
  virtual void startDragging(int)=0;
	void dirUp(); // go one dir up
	void slotTerm();
	void slotEditFile();
	void slotMkDir();
	void slotMoveFiles();
	void slotRenameFile();
	void slotFileProperties(QListViewItem *item=0);
	void slotCopyFiles();
	void slotDeleteFiles();
	void slotPanelRefresh();
  void slotPanelRefresh(const QString path);
  void slotExecuted(QListViewItem *i);
	void home();

  void selectAll();		// mark all the files
  inline void unselectAll() { fileList->unselect(QString("*")); }
  void selectGroup();
  void unselectGroup();
  inline void invertSelection() { fileList->invertSelection(); }
	
  void slotFocusOnMe();		 // give this VFS the focus (the path bar)
	
  	
// these are made non-private to allow easy communications between panels
public:
  class PanelFunc	*func;	
	KVFSPanel		*otherPanel;
	QString 		 virtualPath;
	QString      realPath;
	QToolButton  *bookmarkList;
	QString 		 type;
	vfs					*files;
		
protected:
	bool				 focused;
	QLabel			*status,*totals;
	QPushButton *origin;
	QGridLayout *layout;
	KFileList		*fileList;
	
};



class TreePanel : public KVFSPanel{
friend class TreePanelFunc;
  Q_OBJECT
public:
	// this constructor creates the TREE panel, also needs to be STARTed
  TreePanel(QWidget *parent,const char *name=0);
 ~TreePanel(){}

	void start(bool left);
	// reads the tree from the place marked by PATH
  int readTree(QString path,QListViewItem *father);
  // converts a TREE-item to a full path
  QString returnPath(QListViewItem *item);
 	inline QString getPath(){return returnPath( fileList->currentItem() );}

protected:
  void dragEnterEvent ( QDragEnterEvent * ){}
  void dragLeaveEvent ( QDragLeaveEvent * ){}
  void dragMoveEvent  ( QDragMoveEvent * ){}
  void dropEvent      ( QDropEvent * ){}

public slots:
	void slotRecreateTree(QString path);
  void slotExpandTree(QListViewItem *item);
  void slotCollapsedTree(QListViewItem *item);
  void slotChangeFromTree(QListViewItem *item){
    otherPanel->slotPanelRefresh(otherPanel->realPath=returnPath(item));
    slotFocusOnMe();
  }
  void startDragging  (int){}

protected:

};


class ListPanel : public KVFSPanel{
friend class ListPanelFunc;
	Q_OBJECT
public:
 	// constructor create the panel, but DOESN'T fill it with data, use start()
	ListPanel(QWidget *parent, const bool mirrored, const char *name=0);
 ~ListPanel(){}

	void start(bool left);
  inline QString getPath(){return virtualPath;}
  void cleanUp();

public slots:
	void slotBookmarkChosen(int id);
  void slotRefreshBookmarks();
	void slotUpdateTotals(); // whenever the selection changes
	void slotUpdate();			 // when the vfs finish to update...
	void slotStartUpdate();                   // internal
	void slotEndUpdate();                     // internal
	void slotChangeStatus(QListViewItem *i);  // internal
	void slotGetStats(QString path);          // internal
  void startDragging  (int);                // internal
  void popRightClickMenu(QListViewItem*,QPoint);
protected:
  void dragEnterEvent ( QDragEnterEvent * );// internal
  void dragLeaveEvent ( QDragLeaveEvent * );// internal
  void dragMoveEvent  ( QDragMoveEvent * ); // internal
  void dropEvent      ( QDropEvent * );     // internal
  QListViewItem* itemOn(QPoint p);          // internal

signals:
	void signalStatus(QString msg);

protected:
	QPixmap 	   getIcon(vfile* vf);
	QString      parseSize(unsigned long size);
	QString      shortPath();
	QTimer* t;
	bool canStopTimer;
	
	QPixmap      currDragPix;
	QListViewItem *currDragItem;
	QPopupMenu   *bookmarks;
	QStack<vfs>  vfsStack;
	int currentVfile;
};


#endif


