/***************************************************************************
                                panelfunc.cpp
                             -------------------
    begin                : Thu May 4 2000
    copyright            : (C) 2000 by Shie Erlich & Rafi Yanai
 ---------------------------------------------------------------------------
  Description 
 ***************************************************************************

  A 

     db   dD d8888b. db    db .d8888.  .d8b.  d8888b. d88888b d8888b.
     88 ,8P' 88  `8D 88    88 88'  YP d8' `8b 88  `8D 88'     88  `8D
     88,8P   88oobY' 88    88 `8bo.   88ooo88 88   88 88ooooo 88oobY'
     88`8b   88`8b   88    88   `Y8b. 88~~~88 88   88 88~~~~~ 88`8b
     88 `88. 88 `88. 88b  d88 db   8D 88   88 88  .8D 88.     88 `88.
     YP   YD 88   YD ~Y8888P' `8888Y' YP   YP Y8888D' Y88888P 88   YD

                                                     S o u r c e    F i l e

 ***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "panelfunc.h"
#include <klocale.h>
#include <kio/jobclasses.h>
#include <qmessagebox.h>
#include <qdir.h>
#include <kprocess.h>
#include <kopenwith.h>
#include "krlistitem.h"
#include "krdialogs.h"
#include "normal_vfs.h"
#include "krusader.h"
#include "defaults.h"
#include "arc_vfs.h"

#include <iostream>

//////////////////////////////////////////////////////////
//////		----------	List Panel -------------		////////
//////////////////////////////////////////////////////////

ListPanelFunc::ListPanelFunc(ListPanel *parent):panel(parent){}

void ListPanelFunc::terminal(){
	krConfig->setGroup("External");
	///////////////////////////////
	QString save = getcwd(0,0);
	KShellProcess proc;
	
	proc << krConfig->readEntry("Terminal",_Terminal);
	chdir( panel->realPath );
	proc.start(KProcess::DontCare);
	chdir(save);
}

void ListPanelFunc::editFile(){
	QString name = panel->fileList->currentItem()->text(0);
		
	krConfig->setGroup("Advanced");
	///////////////////////////////
	// verify read permissions
	if(krConfig->readBoolEntry("Permission Check",_PermCheck) &&
	  !panel->files->vfs_search(name)->vfile_isReadable()){
		QMessageBox mb( i18n("Error - Krusader"),
	    i18n("You do not have permission \nto read "+name),
	    QMessageBox::Warning,
	    QMessageBox::Ok | QMessageBox::Default| QMessageBox::Escape,
	    0,0);
    mb.exec();
    return;
	}
	
	if(panel->files->vfs_search(name)->vfile_isDir()){
		QMessageBox mb( i18n("Warning - Krusader"),
	    i18n("You can't edit a directory"),
	    QMessageBox::Warning,
	    QMessageBox::Ok | QMessageBox::Default| QMessageBox::Escape,
	    0,0);
    mb.exec();
    return;
	}
	
	krConfig->setGroup("External");
	///////////////////////////////
  KShellProcess proc;
	proc << krConfig->readEntry("Editor",_Editor) +" \""+
		        panel->files->vfs_getFile(name)+"\"";
  proc.start(KProcess::DontCare);
}

void ListPanelFunc::moveFiles(){
  // check that the current file system support move
  if( !panel->files->supportMoveFrom ){
    QMessageBox mb( i18n("Error - Krusader"),
	    i18n("You can't move files from this file system"),
	    QMessageBox::Warning,
	    QMessageBox::Ok | QMessageBox::Default| QMessageBox::Escape
	    ,0,0);
    mb.exec();
    return;
  }

  // check that the other panel support move to
  if (!panel->otherPanel->files->supportMoveTo){
    QMessageBox mb( i18n("Error - Krusader"),
	    i18n("You can't move files to this file system"),
	    QMessageBox::Warning,
	    QMessageBox::Ok | QMessageBox::Default| QMessageBox::Escape
	    ,0,0);
    mb.exec();
    return;
  }
  // check that the you have write perm
	krConfig->setGroup("Advanced");
	///////////////////////////////
	if(krConfig->readBoolEntry("Permission Check",_PermCheck) &&
	  !panel->files->isWritable){
    QMessageBox mb( i18n("Error - Krusader"),
	    i18n("You do not have write permission to this directory"),
	    QMessageBox::Warning,
	    QMessageBox::Ok | QMessageBox::Default| QMessageBox::Escape
	    ,0,0);
    mb.exec();
    return;
  }

  KURL::List fileUrls;
  KURL url;

	QListViewItem *iterator=panel->fileList->firstChild();
	while (iterator!=0) {
		if (iterator->isSelected() && iterator->text(0)!=QString("..")) {
			krConfig->setGroup("Advanced");
	    ///////////////////////////////
	    // you read & write permission to need a file
	    if(krConfig->readBoolEntry("Permission Check",_PermCheck) &&
		    (!panel->files->vfs_search(iterator->text(0))->vfile_isReadable() ||
		     !panel->files->vfs_search(iterator->text(0))->vfile_isWriteable() )){
		    QMessageBox mb( i18n("Warning - Krusader"),
	        i18n("You do not have permission \nto move "+iterator->text(0)),
	        QMessageBox::Warning,
	        QMessageBox::Ok | QMessageBox::Default| QMessageBox::Escape,
	        QMessageBox::Abort,0);
        switch(mb.exec()){
          case QMessageBox::Abort : return; // stop operation
          case QMessageBox::Ok    : iterator=iterator->itemBelow();
                                  continue;
        }
	    }
		  url.setPath( panel->files->vfs_getFile(iterator->text(0)) );
		  fileUrls.append(url);
		}
		iterator=iterator->itemBelow();
	}
	// if the list id empty - add the current file
	if( fileUrls.count()==0 ){
	  QString name = panel->fileList->currentItem()->text(0);
	  if( name == ".." ) return;
	  if(krConfig->readBoolEntry("Permission Check",_PermCheck) &&
		  (!panel->files->vfs_search(name)->vfile_isReadable() ||
		   !panel->files->vfs_search(iterator->text(0))->vfile_isWriteable() )){
		  QMessageBox mb( i18n("Error - Krusader"),
	      i18n("You do not have permission \nto move "+name),
	      QMessageBox::Warning,
	      QMessageBox::Ok | QMessageBox::Default| QMessageBox::Escape,
	      0,0);
      mb.exec();
      return;
    }
	  url.setPath(panel->files->vfs_getFile(panel->fileList->currentItem()->text(0) ));
	  fileUrls.append(url);
	}	

	QString s;
  if (fileUrls.count()==1 ) s = i18n("Move ")+fileUrls.first().filename()+" "+i18n("to")+":";
	else s.sprintf(i18n("Move %d files to:"),fileUrls.count() );
	// ask the user for the copy dest
	KChooseDir *chooser = new KChooseDir( 0,s,panel->otherPanel->getPath() );
	chooser->setCaption(i18n("Move file(s)"));
	chooser->exec();

	QString dest=chooser->answear;
	if(dest==QString::null) return; // the usr canceled
		
	// if we're not copyning to the other panel
	if(panel->otherPanel->getPath() != dest){
    // you can rename only *one* file not a batch,
		// so a batch dest must alwayes be a directory
		if((fileUrls.count()>1) && (dest.right(1) != "/")) dest = dest+"/";
				  		
		KIO::Job* job = new KIO::CopyJob(fileUrls,dest, false,true,false );
		new KRprogress(job);

  // let the other panel do the dirty job
	}else{
		//check if copy is supported
    if(!panel->otherPanel->files->supportDelete){
      QMessageBox mb( i18n("Error - Krusader"),
	      i18n("You can't copy files to this file system"),
	      QMessageBox::Warning,
	      QMessageBox::Ok | QMessageBox::Default| QMessageBox::Escape
	      ,0,0);
        mb.exec();
        return;
    }
    // check that the you have write perm
	  krConfig->setGroup("Advanced");
	  ///////////////////////////////
	  if(krConfig->readBoolEntry("Permission Check",_PermCheck) &&
	    !panel->otherPanel->files->isWritable){
      QMessageBox mb( i18n("Error - Krusader"),
	      i18n("You do not have write permission to this directory"),
	      QMessageBox::Warning,
	      QMessageBox::Ok | QMessageBox::Default| QMessageBox::Escape
	      ,0,0);
        mb.exec();
        return;
    }
    // finally..
    panel->otherPanel->files->vfs_addFiles(&fileUrls,true,panel);
  }
}

void ListPanelFunc::rename(){
  krConfig->setGroup("Advanced");
  ///////////////////////////////
  QString fileName = panel->fileList->currentItem()->text(0);

  if(krConfig->readBoolEntry("Permission Check",_PermCheck)){
	  if(!panel->files->vfs_search(fileName)->vfile_isWriteable() ||
	     !panel->files->isWritable){
		  QMessageBox mb( i18n("Error - Krusader"),
	      i18n("You do not have permission \nto rename "+fileName),
	        QMessageBox::Warning,
	        QMessageBox::Ok | QMessageBox::Default| QMessageBox::Escape
	        ,0,0);
      mb.exec();
      return;
    }
  } 			

  // ask the new file name
	KQuestion *q = new KQuestion(0,i18n("Rename ")+fileName+i18n(" to:"),fileName);
	q->setCaption(i18n("Rename file"));
	q->exec();

	// if the user canceled - quit
	if (q->answear == QString::null) return;
	// if the name is already taken - quit
	if (panel->files->vfs_search(q->answear) != 0){
		QMessageBox mb( i18n("Can't rename file - Krusader"),
		i18n("A directory or a file with this name\nalready exists "),
		QMessageBox::Warning,
		QMessageBox::Ok | QMessageBox::Default| QMessageBox::Escape,
		0,0 );
    mb.exec();
		return;
	}	
	// as always - the vfs do the job
	panel->files->vfs_rename(fileName,q->answear);

}

void ListPanelFunc::mkdir(){
	// check that the you have write perm
	krConfig->setGroup("Advanced");
	///////////////////////////////
  if(krConfig->readBoolEntry("Permission Check",_PermCheck) &&
	  !panel->files->isWritable){
    QMessageBox mb( i18n("Error - Krusader"),
	    i18n("You do not have write permission to this directory"),
	    QMessageBox::Warning,
	    QMessageBox::Ok | QMessageBox::Default| QMessageBox::Escape
	  ,0,0);
    mb.exec();
    return;
  }
	
	// ask the new dir name..
	KQuestion *q = new KQuestion(0,i18n("Directory's name:"));
	q->setCaption(i18n("New Directory"));
	q->exec();

	// if the user canceled - quit
	if (q->answear == QString::null) return;
	// if the name is already taken - quit
	if (panel->files->vfs_search(q->answear) != 0){
		QMessageBox mb( i18n("Can't create directory - Krusader"),
		i18n("A directory or a file with this file\nalready exists "),
		QMessageBox::Warning,
		QMessageBox::Ok | QMessageBox::Default| QMessageBox::Escape,
		0,0 );
    mb.exec();
		return;
	}	
	// as always - the vfs do the job
	panel->files->vfs_mkdir(q->answear);
}

void ListPanelFunc::copyFiles() {
	KURL::List fileUrls;
  KURL url;
	
	QListViewItem *iterator=panel->fileList->firstChild();
	while (iterator!=0) {
		if (iterator->isSelected() && iterator->text(0)!=QString("..")) {
			krConfig->setGroup("Advanced");
	    ///////////////////////////////
	    // verify write (delete) permissions
	    if(krConfig->readBoolEntry("Permission Check",_PermCheck) &&
		    !panel->files->vfs_search(iterator->text(0))->vfile_isReadable()){
		    QMessageBox mb( i18n("Warning - Krusader"),
	        i18n("You do not have permission \nto copy "+iterator->text(0)),
	        QMessageBox::Warning,
	        QMessageBox::Ok | QMessageBox::Default| QMessageBox::Escape,
	        QMessageBox::Abort,0);
        switch(mb.exec()){
          case QMessageBox::Abort : return; // stop operation
          case QMessageBox::Ok    : iterator=iterator->itemBelow();
                                  continue;
        }
	    }
		  url.setPath( panel->files->vfs_getFile(iterator->text(0)) );
		  fileUrls.append(url);
		}
		iterator=iterator->itemBelow();
	}
	// if the list id empty - add the current file
	if( fileUrls.count()==0 ){
	  QString name = panel->fileList->currentItem()->text(0);
	  if( name == ".." ) return;
	  if(krConfig->readBoolEntry("Permission Check",_PermCheck) &&
		  !panel->files->vfs_search(name)->vfile_isReadable()){
		  QMessageBox mb( i18n("Error - Krusader"),
	      i18n("You do not have permission \nto copy "+name),
	      QMessageBox::Warning,
	      QMessageBox::Ok | QMessageBox::Default| QMessageBox::Escape,
	      0,0);
      mb.exec();
      return;
    }
	  url.setPath( panel->files->vfs_getFile(panel->fileList->currentItem()->text(0)) );
	  fileUrls.append(url);
	}	

	QString s;
  if (fileUrls.count()==1 ) s = i18n("Copy ")+fileUrls.first().filename()+" "+i18n("to")+":";
	else s.sprintf(i18n("Copy %d files to:"),fileUrls.count() );
	// ask the user for the copy dest
	KChooseDir *chooser = new KChooseDir( 0,s,panel->otherPanel->getPath() );
	chooser->setCaption(i18n("Copy file(s)"));
	chooser->exec();

	QString dest=chooser->answear;
	if(dest==QString::null) return; // the usr canceled
		
	// if we're not copyning to the other panel
	if(panel->otherPanel->getPath() != dest){
    // you can rename only *one* file not a batch,
		// so a batch dest must alwayes be a directory
		if((fileUrls.count()>1) && (dest.right(1) != "/")) dest = dest+"/";
				  		
		KIO::Job* job = new KIO::CopyJob(fileUrls,dest, false,false,false );
		new KRprogress(job);

  // let the other panel do the dirty job
	}else{
		//check if copy is supported
    if(!panel->otherPanel->files->supportDelete){
      QMessageBox mb( i18n("Error - Krusader"),
	      i18n("You can't copy files to this file system"),
	      QMessageBox::Warning,
	      QMessageBox::Ok | QMessageBox::Default| QMessageBox::Escape
	      ,0,0);
        mb.exec();
        return;
    }
    // check that the you have write perm
	  krConfig->setGroup("Advanced");
	  ///////////////////////////////
	  if(krConfig->readBoolEntry("Permission Check",_PermCheck) &&
	    !panel->otherPanel->files->isWritable){
      QMessageBox mb( i18n("Error - Krusader"),
	      i18n("You do not have write permission to this directory"),
	      QMessageBox::Warning,
	      QMessageBox::Ok | QMessageBox::Default| QMessageBox::Escape
	      ,0,0);
        mb.exec();
        return;
    }
    // finally..
    panel->otherPanel->files->vfs_addFiles(&fileUrls,false,0);
  }
}

void ListPanelFunc::deleteFiles() {
 	QValueList<QString> fileNames;
 	QValueList<QString>::Iterator name;
  QDir dir;

  //check if delete is supported
  if(!panel->files->supportDelete){
    QMessageBox mb( i18n("Error - Krusader"),
	    i18n("You can't delete files from this file system"),
	    QMessageBox::Warning,
	    QMessageBox::Ok | QMessageBox::Default| QMessageBox::Escape
	    ,0,0);
      mb.exec();
      return;
  }

  // check that the you have write perm
	krConfig->setGroup("Advanced");
	///////////////////////////////
	if(krConfig->readBoolEntry("Permission Check",_PermCheck) &&
	  !panel->files->isWritable){
    QMessageBox mb( i18n("Error - Krusader"),
	    i18n("You do not have write permission to this directory"),
	    QMessageBox::Warning,
	    QMessageBox::Ok | QMessageBox::Default| QMessageBox::Escape
	    ,0,0);
      mb.exec();
      return;
  }

  // first create the name list
	QListViewItem *iterator=panel->fileList->firstChild();
	while (iterator != 0) {
		if(iterator->isSelected() && iterator->text(0)!=QString("..")){
		  fileNames.append(iterator->text(0) );
		}
		iterator = iterator->itemBelow();
	}
	
	// if the list is empty - add current file
	if( fileNames.count() == 0 ){
	  iterator = panel->fileList->currentItem();
	  if(iterator->text(0) == "..") return;
	  else {
	    fileNames.append( iterator->text(0) );
	  }
	}

	// now ask the user if he want to delete:
	QString s;
	if (fileNames.count()==1 )s=fileNames.first()+" ?";
	else s.sprintf( i18n(" %d files ?"),fileNames.count() );
	// show message
	QMessageBox mb( i18n("Warning - Krusader"),
	i18n("Are you sure you want to\ndelete ")+s,
	  QMessageBox::Warning,
	  QMessageBox::Yes | QMessageBox::Default,
	  QMessageBox::No | QMessageBox::Escape,0 );

	// if the user says no - he means no
	if ( mb.exec() == QMessageBox::No ) return;

	//we want to warn the user about non empty dir
	// and files he don't have permission to delete
	for( name = fileNames.begin(); name != fileNames.end(); ){

	krConfig->setGroup("Advanced");
	///////////////////////////////
	  // verify write (delete) permissions
	  if(krConfig->readBoolEntry("Permission Check",_PermCheck) &&
		   !panel->files->vfs_search(*name)->vfile_isWriteable()){
		  QMessageBox mb( i18n("Warning - Krusader"),
	      i18n("You do not have permission \nto delete "+*name),
	      QMessageBox::Warning,
	      QMessageBox::Ok | QMessageBox::Default| QMessageBox::Escape,
	      QMessageBox::Abort,0);
      switch(mb.exec()){
        case QMessageBox::Abort : return; // stop operation
        case QMessageBox::Ok    : name = fileNames.remove(name);
                                  continue;
      }
	  }

    // verify non-empty dirs delete... (only for norml vfs)
		if ( panel->files->vfs_getType() == "normal" &&
		     panel->files->vfs_search(*name)->vfile_isDir() ){
		
		  dir.setPath( panel->getPath()+"/"+(*name) );
			if ( dir.count() > 2 && krConfig->readBoolEntry("Confirm Unempty Dir",_ConfirmUnemptyDir) ){
			  switch(QMessageBox::warning( 0,i18n("Warning - Krusader"),
									i18n("Directory "+(*name)+" is not empty !\nAre you sure ? "),
									i18n("&Skip"),i18n("S&ure"),i18n("&Abort"),0,2 )){
			    case 2: return;
				  case 1:	++name;
				          continue;
				  case 0: name = fileNames.remove(name);
                  continue;
			  }
			}
		}
		++name;
	}

  if(fileNames.count() == 0) return;
  // let the vfs do the job...
	panel->files->vfs_delFiles(&fileNames);
}

// this is done when you double click on a file
void ListPanelFunc::execute(QListViewItem *i) {
	QString name=i->text(0);
	if(name==".."){
		dirUp();
		return;
	}
	vfile *vf=panel->files->vfs_search(name);
	if(vf == 0) return;
	cout << "mime = "+vf->vfile_getMime()<<endl;
	QString origin= panel->files->vfs_getOrigin();
	
	krConfig->setGroup("Advanced");
	///////////////////////////////
	// verify read permissions
	if(krConfig->readBoolEntry("Permission Check",_PermCheck)){
    if(!vf->vfile_isReadable()){
      QMessageBox mb( i18n("Error - Krusader"),
	         i18n("You don't have premission to\naccess this Directory / File !"),
	         QMessageBox::Warning,
	         QMessageBox::Ok | QMessageBox::Default| QMessageBox::Escape
	         ,0,0);
        mb.exec();
        return;
    }
	}
	
	if (vf->vfile_isDir()){
	  origin=="/"? origin+=name : origin+="/"+name;
		refresh(origin);
	}
	else{
	  krConfig->setGroup("External");
	  QString type = vf->vfile_getMime().right(4);
	  if((type == "-tgz" && krConfig->readBoolEntry("Do GZip" ,_DoGZip )  ) ||
	     (type == "-tar" && krConfig->readBoolEntry("Do Tar"  ,_DoTar  )  ) ||
	     (type == "-tbz" && krConfig->readBoolEntry("Do BZip2",_DoBZip2)  ) ||
	     (type == "gzip" && krConfig->readBoolEntry("Do GZip" ,_DoGZip )  ) ||
	     (type == "zip2" && krConfig->readBoolEntry("Do BZip2",_DoBZip2)  ) ||
	     (type == "-zip" && krConfig->readBoolEntry("Do UnZip",_DoUnZip)  ) ||
	     (type == "-rpm" ) )
	    changeVFS(vf,type);
	  else{
	    new KFileOpenWithHandler();
	    KURL url;
	    url.setPath(panel->files->vfs_getFile(name));
		  new KRun(url,vf->vfile_getMode() );
    }
  }
}

void ListPanelFunc::refresh(const QString path){
 	// first, make this panel the active one
	panel->slotFocusOnMe();
	// second, make the status line empty -> needed for correct flow
	panel->status->setText("");
	// make sure we're in the right vfs..
	if (panel->virtualPath.contains('\\') && !path.contains('\\')){
	  while(!panel->vfsStack.isEmpty()){
      delete panel->files;
      panel->files = panel->vfsStack.pop();
    }
    panel->files->active = true;
  }
	// if we could not refresh try to dir up
	QString origin = path;
	if (!panel->files->vfs_refresh(origin)) {
	  panel->virtualPath = origin;
	  dirUp();
	}
}

void ListPanelFunc::dirUp(){
	QString origin= panel->virtualPath;
	if (origin=="/") return;
	
	//do we need to change vfs ?
	if( origin.right(1) == "\\" ){ //yes
	  delete panel->files;
	  panel->files = panel->vfsStack.pop();
	  panel->files->active = true;
	  connect(panel->files,SIGNAL(startUpdate()),panel,SLOT(slotStartUpdate()));
	  connect(panel->files,SIGNAL(updated()),panel,SLOT(slotUpdate()));
	}
	
	// remove one dir from the path
	origin.truncate(origin.findRev('/'));
	// check the '/' case
	if (origin=="") origin="/";
	
	// change dir..
	refresh(origin);
}

void ListPanelFunc::changeVFS(vfile* vf,QString type){
  // save the current vfs
  panel->files->active = false;
  panel->vfsStack.push(panel->files);

  QString origin = panel->files->vfs_getFile(vf->vfile_getName());

  panel->files = new arc_vfs(origin,type,panel,panel->files->isWritable);
  panel->files->vfs_refresh(panel->virtualPath+"/"+vf->vfile_getName()+"\\");
}

void ListPanelFunc::selectAll() {
	panel->fileList->select(QString("*"));
}

void ListPanelFunc::selectGroup() {
  KQuestion *q = new KQuestion(0,i18n("Select the following files"),"*");
	q->setCaption(i18n("Select Files"));
	q->exec();
	// if the user canceled - quit
	if (q->answear == QString::null) return;
	panel->fileList->select(q->answear);
}

void ListPanelFunc::unselectGroup() {
  KQuestion *q = new KQuestion(0,i18n("Unselect the following files"),"*");
	q->setCaption(i18n("Unselect Files"));
	q->exec();
	// if the user canceled - quit
	if (q->answear == QString::null) return;
	panel->fileList->unselect(q->answear);
}

//////////////////////////////////////////////////////////
//////		----------	Tree Panel -------------		////////
//////////////////////////////////////////////////////////

TreePanelFunc::TreePanelFunc(TreePanel *parent):panel(parent){}

void TreePanelFunc::dirUp(){}
void TreePanelFunc::terminal(){}
void TreePanelFunc::editFile(){}
void TreePanelFunc::mkdir(){}
void TreePanelFunc::rename(){}

void TreePanelFunc::moveFiles(){}
void TreePanelFunc::copyFiles(){}
void TreePanelFunc::deleteFiles(){}
void TreePanelFunc::execute(QListViewItem *i){}
void TreePanelFunc::refresh(const QString path){}

void TreePanelFunc::selectAll(){}	// not implemeted for trees
void TreePanelFunc::selectGroup(){}	// not implemeted for trees
void TreePanelFunc::unselectGroup(){}	// not implemeted for trees

