/***************************************************************************
                          vfile.h
                      -------------------
    Version							 : Milestone 1
    begin                : Thu May 4 2000
    copyright            : (C) 2000 by Shie Erlich & Rafi Yanai
 ---------------------------------------------------------------------------
 * the Virtual File class handles all the details of maintaining a single  *
 * file component within the virtual file system (vfs). a vfile object     *
 * contains the nessecery details about a file and member functions which  *
 * allow the object to give out the needed details about the file.         *
 ***************************************************************************

  A

     db   dD d8888b. db    db .d8888.  .d8b.  d8888b. d88888b d8888b.
     88 ,8P' 88  `8D 88    88 88'  YP d8' `8b 88  `8D 88'     88  `8D
     88,8P   88oobY' 88    88 `8bo.   88ooo88 88   88 88ooooo 88oobY'
     88`8b   88`8b   88    88   `Y8b. 88~~~88 88   88 88~~~~~ 88`8b
     88 `88. 88 `88. 88b  d88 db   8D 88   88 88  .8D 88.     88 `88.
     YP   YD 88   YD ~Y8888P' `8888Y' YP   YP Y8888D' Y88888P 88   YD

                                                     H e a d e r    F i l e

 ***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#ifndef VFILE_H
#define VFILE_H

// QT includes
#include <qstring.h>
#include <qintdict.h>
// KDE includes
#include <kmimetype.h>
// System includes
#include <sys/types.h>
#include <unistd.h>

class vfile{
private:
	// the file information list
	QString 	    vfile_name;		  // name
	unsigned long vfile_size;     // size
	QString				vfile_owner;		// owner name
	QString 			vfile_group;		// group name
	mode_t				vfile_mode;			// the vfile mode
	QString			 	vfile_ownerName;// owner name
	QString				vfile_groupName;// group name
	QString     	vfile_perm;			// permissions
	QString			  vfile_dateTime;	// modification date&time
	bool					vfile_symLink;  // true==yes
	QString				vfile_mimeType; // file mimetype
	QString				vfile_symDest;  // if it's a sym link - its detination
	
public:
  // the constractor needs:
	// (file name, file size, file permissions,is the file a link,
	// owner uid, group uid)
	vfile(QString name,	
				unsigned long size,	
			  QString perm,
				QString	dateTime,
				bool symLink,
				uid_t	owner,
				gid_t group,
				QString mime,
				QString symDest,
				mode_t  mode);
	
	vfile(QString name,	
				unsigned long size,	
			  QString perm,
				QString	dateTime,
				bool symLink,
				QString	owner,
				QString group,
				QString mime,
				QString symDest,
				mode_t  mode);
	
	// following functions give-out file details
	inline QString  			 	vfile_getName() 	 	{ return vfile_name;  		}
	inline unsigned long		vfile_getSize() 	 	{ return vfile_size;  		}
//	inline QString				 	vfile_getOwner()	  { return vfile_owner; 		}
//	inline Qstring				 	vfile_getGroup()	  { return vfile_group; 		}
	inline QString 		  		vfile_getDateTime() { return vfile_dateTime;	}
	inline QString					vfile_getPerm()			{ return vfile_perm;			}
	inline bool							vfile_isDir()				{ return (vfile_perm[0]=='d');}
	inline bool							vfile_isSymLink()		{ return vfile_symLink;   }
	inline QString					vfile_getMime()			{ return vfile_mimeType;	}
	inline QString					vfile_getSymDest()	{ return vfile_symDest;		}
	inline mode_t						vfile_getMode()			{ return vfile_mode;			}			
				 bool							vfile_isReadable();
	       bool 						vfile_isWriteable();
  			 bool							vfile_isExecutable();
				
	
	// boolean operator assume no 2 files have the same name !
  inline bool operator==(const vfile& vf){ return (vf.vfile_name==vfile_name); }
  inline bool operator!=(const vfile& vf){ return (vf.vfile_name!=vfile_name); }
  inline bool operator<=(const vfile& vf){ return (vf.vfile_name<=vfile_name); }
  inline bool operator>=(const vfile& vf){ return (vf.vfile_name>=vfile_name); }
	inline bool operator< (const vfile& vf){ return (vf.vfile_name< vfile_name); }
	inline bool operator> (const vfile& vf){ return (vf.vfile_name> vfile_name); }

private:
	// cache for passwd and group entries
	typedef QIntDict<char> IntCache;
	static IntCache *passwdCache;
  static IntCache *groupCache;		
  void initCaches();
};

#endif
