/***************************************************************************
	  	                        vfs.h
  	                    -------------------
    Version							 : Milestone 1
    begin                : Thu May 4 2000
    copyright            : (C) 2000 by Shie Erlich & Rafi Yanai
 *-------------------------------------------------------------------------*
 * the vfs class is an extendable class which by itself does (almost)      *
 * nothing. other VFSs like the normal_vfs inherits from this class and    *
 * make it possible to use a consistent API for all types of VFSs.         *
 *                                                                         *
 ***************************************************************************

   A 

     db   dD d8888b. db    db .d8888.  .d8b.  d8888b. d88888b d8888b.
     88 ,8P' 88  `8D 88    88 88'  YP d8' `8b 88  `8D 88'     88  `8D
     88,8P   88oobY' 88    88 `8bo.   88ooo88 88   88 88ooooo 88oobY'
     88`8b   88`8b   88    88   `Y8b. 88~~~88 88   88 88~~~~~ 88`8b
     88 `88. 88 `88. 88b  d88 db   8D 88   88 88  .8D 88.     88 `88.
     YP   YD 88   YD ~Y8888P' `8888Y' YP   YP Y8888D' Y88888P 88   YD

                                                     H e a d e r    F i l e

 ***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef VFS_H
#define VFS_H

// Krusader includes
#include "krdialogs.h"
#include "vfile.h"

// QT includes
#include <qstring.h>
#include <qlist.h>
#include <qobject.h>

// KDE includes
#include <kurl.h>

class vfs: public QObject{
	Q_OBJECT
protected: // allows derived classes to use these fields
	QString       vfs_origin;	 	// the path or file the VFS originates from
	QString		  	vfs_type;			// the vfs type;						
 //	QList<vfile>  vfs_files;    // list of pointers to vfile
 	QList<vfile> *vfs_filesP;
 	
public: 	
 	KReadVFS		*waitDlg;			// the PleaseWait box...
 	bool active;              // true if the vfs is active
 	
 	bool supportCopyTo;       // true if you can add files
 	bool supportMoveTo;       // true if you can move files to
 	bool supportMoveFrom;     // true if you can move files out
 	bool supportDelete;       // true if you can support delete
 	bool isWritable;	        // true if it's writable
	
	// service functions
  void    vfs_addToList(vfile *data);
  void    vfs_removeFromList(vfile *data);
  QString dateTime2QString(const QDateTime& datetime);
  QString time2QString(long time);
  QString mode2QString(mode_t m);
	
public:
  // create a vfs from origin
		vfs(QWidget* panel):waitDlg(0),active(true){
		 // vfs_files.setAutoDelete(true); // delete items when they are removed
      connect(this,SIGNAL(updated()),panel,SLOT(slotUpdate()));
      connect(this,SIGNAL(startUpdate()),panel,SLOT(slotStartUpdate()));
      connect(this,SIGNAL(endUpdate()),panel,SLOT(slotEndUpdate()));
   }

	virtual			~vfs(){}
	// copy a file to the vfs (physical)
	virtual void vfs_addFiles(KURL::List *fileUrls,bool move,QWidget* toNotify,QString dir = "")=0;	
	// remove a file from the vfs (physical)
	virtual void vfs_delFiles(QValueList<QString> *fileNames)=0;	
	// return a path to the file
	virtual QString vfs_getFile(QString name)=0;
	// make dir
	virtual void vfs_mkdir(QString name)=0;
	// rename file
	virtual void vfs_rename(QString fileName,QString newName)=0;
	
	long vfs_totalSize();             // the total size of FILES in the vfs,
	vfile* vfs_search(QString name);  // return vfile* or 0 if not found
	
	inline   int  vfs_noOfFiles() 	 	{ return vfs_filesP->count(); 	}
	inline	 QString vfs_getOrigin() 	{ return vfs_origin; 					  }	
	inline 	 QString vfs_getType()	 	{ return vfs_type; 						  }
	inline   vfile* vfs_getFirstFile(){ return (vfs_filesP->first());	}
	inline   vfile* vfs_getNextFile()	{	return (vfs_filesP->next());  }
	inline   vfile* vfs_at(int i)     { return (vfs_filesP->at(i));   }

public slots:
	// actually reads files and stats
	virtual bool vfs_refresh(const QString& origin)=0;
	inline  bool vfs_refresh(){return vfs_refresh(vfs_origin);}

signals: 	
  void updated();
  void startUpdate();
  void endUpdate();
};

#endif
